package com.almworks.jira.structure.apiclient;

import com.almworks.jira.structure.api.StructureComponents;
import com.almworks.jira.structure.api.error.StructureException;
import com.almworks.jira.structure.api.query.StructureQuery;
import com.almworks.jira.structure.api.query.StructureQueryParser;
import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.workflow.WorkflowException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;

public class UserStructureQueryCondition extends AbstractStructureQueryCondition {
  private static final Logger log = LoggerFactory.getLogger(UserStructureQueryCondition.class);
  private final StructureQueryParser myStructureQueryParser;

  public UserStructureQueryCondition(StructureComponents structureComponents) {
    super(structureComponents.getStructureManager());
    myStructureQueryParser = structureComponents.getStructureQueryParser();
  }

  @Override
  public boolean passesCondition(Map transientVars, Map args, PropertySet ps) throws WorkflowException {
    Object structureQueryObj = args.get(UserStructureQueryConditionFactory.STRUCTURE_QUERY_KEY);
    if (!(structureQueryObj instanceof String)) {
      log.warn("No structure query, ignoring condition");
      return true;
    }
    try {
      StructureQuery query = myStructureQueryParser.parse((String) structureQueryObj);
      return matchesIssue(query, transientVars, args);
    } catch (StructureException ignored) {
      return true;
    }
  }
}
