package com.almworks.jira.structure.apiclient;

import com.almworks.jira.structure.api.StructureComponents;
import com.almworks.jira.structure.api.error.StructureException;
import com.almworks.jira.structure.api.query.StructureQuery;
import com.almworks.jira.structure.api.query.StructureQueryParser;
import com.atlassian.jira.plugin.workflow.AbstractWorkflowPluginFactory;
import com.atlassian.jira.plugin.workflow.WorkflowPluginConditionFactory;
import com.atlassian.jira.util.MessageSet;
import com.google.common.collect.Maps;
import com.opensymphony.workflow.loader.AbstractDescriptor;
import com.opensymphony.workflow.loader.ConditionDescriptor;

import java.util.List;
import java.util.Map;

import static com.google.common.collect.Lists.newArrayList;

public class UserStructureQueryConditionFactory extends AbstractWorkflowPluginFactory implements WorkflowPluginConditionFactory {
  public static final String STRUCTURE_QUERY_KEY = "structureQuery";
  public static final String STRUCTURE_QUERY_VALID = "structureQueryValid";
  public static final String STRUCTURE_QUERY_INVALID_STRING_KEY = "structureQueryInvalidString";
  public static final String STRUCTURE_QUERY_ERRORS_KEY = "structureQueryErrors";
  private final StructureQueryParser myStructureQueryParser;

  public UserStructureQueryConditionFactory(StructureComponents structureComponents) {
    myStructureQueryParser = structureComponents.getStructureQueryParser();
  }

  @Override
  protected void getVelocityParamsForInput(Map<String, Object> velocityParams) {    
  }

  @Override
  protected void getVelocityParamsForEdit(final Map<String, Object> velocityParams, AbstractDescriptor descriptor) {
    if (descriptor instanceof ConditionDescriptor) {
      Map args = ((ConditionDescriptor) descriptor).getArgs();

      if ("true".equals(args.get(STRUCTURE_QUERY_VALID))) {
        velocityParams.put(STRUCTURE_QUERY_KEY, args.get(STRUCTURE_QUERY_KEY));
        
        // Remove error messages from the arguments
        args.remove(STRUCTURE_QUERY_ERRORS_KEY);
        args.remove(STRUCTURE_QUERY_INVALID_STRING_KEY);
      } else {
        Object structureQueryErrorsObj = args.get(STRUCTURE_QUERY_ERRORS_KEY);
        if (structureQueryErrorsObj != null) {
          if (!(structureQueryErrorsObj instanceof List)) {
            structureQueryErrorsObj = newArrayList(structureQueryErrorsObj);
          }
          velocityParams.put(STRUCTURE_QUERY_ERRORS_KEY, (List) structureQueryErrorsObj);
        }
        
        velocityParams.put(STRUCTURE_QUERY_INVALID_STRING_KEY, args.get(STRUCTURE_QUERY_INVALID_STRING_KEY));
      }
    }
  }

  @Override
  protected void getVelocityParamsForView(Map<String, Object> velocityParams, AbstractDescriptor descriptor) {
    getVelocityParamsForEdit(velocityParams, descriptor);
  }

  @Override
  public Map<String, ?> getDescriptorParams(Map<String, Object> formParams) {
    Map<String, Object> args = Maps.newHashMap();
    String sjql = extractSingleParam(formParams, "sjql");
    try {
      StructureQuery structureQuery = myStructureQueryParser.parse(sjql);
      MessageSet messages = structureQuery.validate();
      if (!messages.hasAnyErrors()) {
        args.put(STRUCTURE_QUERY_KEY, structureQuery);
        args.put(STRUCTURE_QUERY_VALID, true);
      } else {
        args.put(STRUCTURE_QUERY_ERRORS_KEY, newArrayList(messages.getErrorMessages()));
        args.put(STRUCTURE_QUERY_INVALID_STRING_KEY, sjql);
        args.put(STRUCTURE_QUERY_VALID, false);
      }
    } catch (StructureException e) {
      args.put(STRUCTURE_QUERY_ERRORS_KEY, newArrayList(e.getLocalizedMessage()));
      args.put(STRUCTURE_QUERY_INVALID_STRING_KEY, sjql);
      args.put(STRUCTURE_QUERY_VALID, false);
    }
    return args;
  }
}
